#include "simpletools.h"
#include "propeller.h"

/*  CY62256N
 *  256-Kbit (32 K × 8) SRAM
 *  70ns
 *  Propeller Flip rev.a
 *  P0-7 = I/O 0-7
 *  P8   = WE write enable  active low
 *  P9   = OE output enable active low
 *  P10  = A0  = C0
 *  P11  = A1  = C1
 *  P12  = A11 = C2
 *  P13  = A12 = C3
 *  P14  = A13 = C4
 *  P15  = A14 = C5
 *  P16  = A2  = R0
 *  P17  = A3  = R1
 *  P18  = A4  = R2
 *  P19  = A5  = R3
 *  P20  = A6  = R4
 *  P21  = A7  = R5
 *  P22  = A8  = R6
 *  P23  = A9  = R7
 *  P24  = A10 = R8
 *  P25  = CE chip enable   active low
 */

void init_RAM();
void open_write();
void close_write();
void open_read();
void close_read();
void write_byte(uint32_t byte_out, uint32_t address);
uint32_t read_byte(uint32_t address);
void read_row(uint32_t array_in, uint32_t row);

int main() {                  // Entry point
  init_RAM();
  open_write();       // Open write
  uint32_t j, start, end, reg, data[64];
 
  j = 0;
  while(j < 16) {
    data[j] = 0;
    j++;
  }     
  
  j = 0;
  start = CNT;        // Start timing write byte
  while(j < 32768) {
    write_byte(j, j); // Write byte
    j++;
  }
  end = CNT;          // Stop timing write byte
  print("%d random byte writes per second\n", CLKFREQ / ((end-start) / 32768));    
  close_write();      // Close write
  
  open_read();
  j = 0;
  while(j < 511) {
    print("%d: %d\n", j, read_byte(j));
    j++;
  }
  
  j = 0;
  start = CNT;        // Start timing read byte 
  while(j < 32768) {
    reg = read_byte(j); // Read byte and store in reg
    j++;
  }
  end = CNT;          // Stop timing read byte
  close_read(); 
   
  print("%d random byte reads per second\n", CLKFREQ / ((end-start) / 32768)); 


    j=0;
    open_read();      // Open read
    start = CNT;      // Start timing read row
    while(j < 64) {
      read_row(data, j);  // Read row
      j++;
    }      
    end = CNT;        // Stop timing read row
    close_read();     // Close read

    
  j = 0;
  while(j < 64) { // Read 64 rows of 64 bytes each
    print("%d\n", ((uint8_t *)(data))[j]);
    j++;
  }
  
  print("%d sequential byte reads per second\n", CLKFREQ / ((end-start) / 4096));    
   
}

void init_RAM() {
  DIRA |= (262143 << 8);      // Set address and control bits to outgoing
  OUTA &= ~67108863;          // Clear address, control, and bus bits
  OUTA |= (1 << 25);          // Raise CE to disable
  OUTA |= (1 << 8);           // Raise WE to prevent write
  OUTA |= (1 << 9);           // Raise OE to prevent read
}

void open_write() {
  DIRA |= 255;                // Set bus to outgoing
  OUTA = (OUTA & 0xfe0003ff) | ((0 & 0x7fff) << 10);  // Clear address
  OUTA &= ~(1 << 25);         // Lower CE to enable
}

void close_write() {
  OUTA |= (1 << 25);          // Raise CE to disable
  OUTA &= ~255;               // Clear bus
}

__attribute__((fcache)) void write_byte(uint32_t byte_out, uint32_t address) {
  OUTA = (OUTA & 0xfe0003ff) | ((address & 0x7fff) << 10);  // Input address
  OUTA = (OUTA & 0xffffff00) | ((byte_out & 0xff) << 0);    // Write data Byte to bus
  OUTA &= ~(1 << 8);          // Lower WE to begin write sequence
  OUTA |= (1 << 8);           // Raise WE to end write sequence
}

void open_read() {
  DIRA &= ~255;               // Set bus to incoming
  OUTA = (OUTA & 0xfe0003ff) | ((0 & 0x7fff) << 10);  // Clear address
  OUTA &= (~(1 << 25));       // Lower CE to enable
  OUTA &= (~(1 << 9));        // Lower OE to begin read sequence
}

void close_read() {
  OUTA |= (1 << 9);           // Raise OE to end read sequence
  OUTA |= (1 << 25);          // Raise CE to disable
  OUTA &= ~255;               // Clear bus
}

__attribute__((fcache)) uint32_t read_byte(uint32_t address) {
  uint32_t result = 0;          
  OUTA = (OUTA & 0xfe0003ff) | ((address & 0x7fff) << 10);  // Input address
  OUTA &= (~(1 << 9));        // Lower OE to begin read sequence
  result = INA & 0xff;        // Record data Byte from bus
  OUTA |= (1 << 9);           // Raise OE to end read sequence
  return result;
}

//Array by name, address of first byte, length in bytes
__attribute__((fcache)) void read_row(uint32_t array_in, uint32_t row) {    
  row = row<<6;
  for(int32_t i = 0; i < 64; i++) {
    OUTA = (OUTA & 0xfe0003ff) | (((row + i) & 0x7fff) << 10);    // Input address
    ((uint8_t *)(array_in))[i] = INA & 0xff;                      // Record data Byte from bus
  } 
}